#define __SFR_OFFSET 0

#include <avr/io.h>

#if defined(__AVR_ATmega328PB__)

;sync interrupt
;An interrupt is generated at each horizontal sync
;We process the pulses to get the vertical sync and field
.global ANALOG_COMP_vect
ANALOG_COMP_vect:
        push    r16
        push    r17

        ;save status register
        in      r16,SREG
        push    r16

        ;to detect the vertical sync, we check the state of the
        ;comparator output about 20 cycles after the interrupt.
        push    r15
        lds     r15,sync_counter

        ;to detect the even/odd field, the timer0 counts at 1MHz rate
        ;and we read the count value

        lds     r16,pipeline
        in      r17,TCNT0       ;get the number of pulses before vertical
        ;measured
        ;>49 for full field -->  >32 >0x20 20-FF
        ;<20 for half field -->  <32 <0x20 00-1F
        ;store bit 5 in bit-pipeline  0x20
		;get bit in carry
        lsl     r17         ;0x40
        lsl     r17         ;0x80
        lsl     r17         ;CY
        ;store bit in bit-pipeline
        rol     r16
		
	;save pipeline byte for next interrupt
        sts     pipeline,r16
        
        ;reset T0 count for next interrupt
        clr     r16
        out     TCNT0,r16
        
		    ;delay
		    ldi		  r16,10
1:	    dec		  r16
		    brne	  1b

        ;if the ACO line is high, the first pulse clears the counter
        ;R15 is the flag for resetting on the 1st pulse.
        
        ;if (ACSR.ACO == 1)
		    in		r16,ACSR
		    sbrc	r16,ACO
        rjmp  2f
            tst		r15
            brne  3f
                ;Each field
                lds   r16,pipeline  ;check for odd/even field

                lsr   r16
                lsr   r16
                lsr   r16
                out   PORTD,r16
                
                ;At this point, we get the state of the bit from the 6th previous interrupt
                sbrs  r16,3   ;6
                jmp   1f
                    ;odd field
                    clr   r16         ;reset the counter on odd fields
                    ;ldi   r16,0x03
                    sts   TCNT4H,r16
                    ;ldi   r16,0xff
                    sts   TCNT4L,r16
                    ;sbi   TIFR4,1
1:          
                inc   r15
                rjmp  3f
        ;else
2:	        clr   r15
        ;endif
3:
        sts     sync_counter,r15
        pop     r15
        
        ;restore status register
        pop     r16
        out     SREG,r16
        pop     r17
        pop     r16
        reti
#endif  /* __AVR_ATmega328PB__ */
